@@ -2,28 +2,19 @@ package Gimp::Lib;
 
 use strict;
 use Carp;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD @_consts @_procs %EXPORT_TAGS @EXPORT_FAIL);
-use Gimp;
+use vars qw($VERSION @ISA);
 
 require DynaLoader;
 
 @ISA = qw(DynaLoader);
 $VERSION = $Gimp::VERSION;
 
-use subs @Gimp::_procs;
-use subs @Gimp::_internals;
+use subs qw(
+	gimp_call_procedure		gimp_main
+	_gimp_procedure_available	set_trace
+);
 
-sub AUTOLOAD {
-   my $constname;
-   ($constname = $AUTOLOAD) =~ s/.*:://;
-   if (_gimp_procedure_available ($constname)) {
-      no strict "refs";
-      *{$AUTOLOAD} = sub { gimp_call_procedure $constname,@_ };
-      goto &$AUTOLOAD;
-   } else {
-      croak "$constname not defined in Gimp";
-   }
-}
+bootstrap Gimp::Lib $VERSION;
 
 # functions to "autobless" where the autobless mechanism
 # does not work.
@@ -40,8 +31,6 @@ sub gimp_image_get_channels {
    map _autobless($_,&Gimp::PARAM_CHANNEL),gimp_call_procedure "gimp_image_get_channels",@_;
 }
 
-bootstrap Gimp::Lib $VERSION;
-
 sub gimp_gdrawable_width	{ $_[0]->{_width}	}
 sub gimp_gdrawable_height	{ $_[0]->{_height}	}
 sub gimp_gdrawable_ntile_rows	{ $_[0]->{_ntile_rows}	}
@@ -78,9 +67,9 @@ Gimp::Lib - Interface to libgimp (as opposed to Gimp::Net)
 
 =head1 DESCRIPTION
 
-This is package that uses libgimp to interface with the Gimp, i.e. the
-normal interface to use with the Gimp. You don't normally use this module
-directly, look at the documentation for the package "Gimp".
+This is the package that interfaces to The Gimp via the libgimp interface,
+i.e. the normal interface to use with the Gimp. You don't normally use this
+module directly, look at the documentation for the package "Gimp".
 
 =head1 AUTHOR
 
